--[[ 
Bill's Utils

This addon contains functions used in several of my addons so in order to avoid duplicate code
I moved them to this utility library 

This library will wipe out it's globals after 5 mins so make sure to reference them in local variables
in the addons that need them any unreferenced functions will be garbage collected afterwards to minimize
the memory footprint ]]

local on = true
local off = false
local X = 16 -- standard Horizontal Spacing
local Y = 35 -- standard Vertical spacing 26
local TxtY = 13


BillsUtils = {}

-- Wait/Delay function
local waitTable = {};
local waitFrame = nil;

BillsUtils.Wait = function(delay, func, ...)
  if(type(delay)~="number" or type(func)~="function") then
    return false;
  end
  if(waitFrame == nil) then
    waitFrame = CreateFrame("Frame","WaitFrame", UIParent);
    waitFrame:SetScript("onUpdate",function (self,elapse)
      local count = #waitTable;
      local i = 1;
      while(i<=count) do
        local waitRecord = tremove(waitTable,i);
        local d = tremove(waitRecord,1);
        local f = tremove(waitRecord,1);
        local p = tremove(waitRecord,1);
        if(d>elapse) then
          tinsert(waitTable,i,{d-elapse,f,p});
          i = i + 1;
        else
          count = count - 1;
          f(unpack(p));
        end
      end
    end);
  end
  tinsert(waitTable,{delay,func,{...}});
  return true;
end

-- Add Meta Tables to Variables
BillsUtils.addOptionMt = function(options, defaults)
	setmetatable(options, {__index = defaults})
	for i, v in pairs(options) do
		if type(v) == "table" and not getmetatable(v) then
			addOptionMt(v, defaults[i])
		end
	end
end

-- Return Colored status
BillsUtils.StatColor = function(stat)
	local vtype = type(stat)
	if vtype == "boolean" then
		if stat == on then
			return "|cFF00FF00[on]"
		elseif stat == off then
			return "|cFFFF0000[off]"
		end
	elseif vtype == "number" then
		if stat == 0 then
			return "|cFFFF0000[0] (DISABLED)"
		elseif stat > 0 and stat <= 60 then
			return string.format("|cFF00FF00[%i] second(s)", stat)
		end
	end
end

-- String Join print
BillsUtils.SJprint = function(...)
	print( string.join(" ", ...))
end

-- Sorts 2 tables where x in both tables are related
BillsUtils.LinkedSort = function( main, linked )
	if #main == 0 or # linked == 0 then
		return false
	end
	local sorted = false
	
	while sorted == false do
		local changed = false
		for x = #main -1, 1, -1 do
			if main[x] > main[x+1] then
				main[x], main[x+1] = main[x+1], main[x]
				linked[x], linked[x+1] = linked[x+1], linked[x]
				changed = true
			end
		end
		if not(changed) then
			sorted = true
		end
	end	
end

-- ****************************************************************************
-- * Cmd Line Utils                                                           *
-- ****************************************************************************

-- Checks CmdLine switches agains variable and changes var if needed true/false
BillsUtils.OptSaveTF = function( Table, Key, Value)
	local Saved = Table[ Key ]
	if Value == "on" or Value == "off" then
		if Saved ~= Value then
			Table[ Key ] = Value == "on"
		end
	end
end

-- Checks CmdLine switches agains variable and changes var if needed Number
BillsUtils.OptSaveVal = function( Table, Key, Value, Min, Max)
	local Saved = Table[ Key ]
	if Value == nil then
		return
	end
	if Value ~= Saved then
		if Value >= Min and Value <= Max then
			Table[ Key ] = Value
		end
	end
end

-- ****************************************************************************
-- * Check Box Utils                                                          *
-- ****************************************************************************

BillsUtils.ChkBox = function( Last , Prefix, Name, Parent, Click, TextTable )
	local box
	box = CreateFrame( "CheckButton", Prefix..Name.."Checkbox", Parent, "InterfaceOptionsCheckButtonTemplate" )
	if Last == Parent then
		box:SetPoint("TOPLEFT", Last ,"TOPLEFT", X*1.25 , -Y/2 )
	else
		box:SetPoint("TOPLEFT", Last ,"TOPLEFT", 0, -Y )
	end
	box:SetSize( 26, 26 )
	box:SetScript( "OnClick", Click )
	
	box.Text = Parent:CreateFontString( nil, "ARTWORK", "GameFontHighlightSmall" )
	box.Text:SetPoint( "BOTTOMLEFT", box, "BOTTOMLEFT", X*2 , -TxtY )
	box.Text:SetPoint( "RIGHT", -32, 0 )
	box.Text:SetHeight( 32 )
	box.Text:SetJustifyH( "LEFT" )
	box.Text:SetJustifyV( "TOP" )
	box.Text:SetText( TextTable[ Name ] )
	return box
end

BillsUtils.CheckClick = function()
	PlaySound( "GAMEGENERICBUTTONPRESS" )
end

-- Checks checkboxes against variable and adjusts variable if needed
BillsUtils.ChkBoxSave = function( Panel, Box, Table, Key)
	if Key == nil then
		Key = Box
	end
	
	local checked = Panel[ Box ]:GetChecked() == 1
	local saved = Table[Key]
	
	if checked ~= saved then
		Table[ Key ] = checked
	end
end

-- ****************************************************************************
-- * Slider Utils                                                             *
-- ****************************************************************************

BillsUtils.Slider = function( Last , Prefix, Name, Parent, SliderMW , TextTable, min, max, step )
	local slider
	slider = CreateFrame( "Slider", Prefix..Name.."Slider", Parent, "OptionsSliderTemplate" )
	slider:SetMinMaxValues( min, max )
	slider:SetOrientation("HORIZONTAL")
	slider:SetPoint("TOPLEFT", Last ,"TOPLEFT", 0 , -Y * 1.25)
	slider:SetValueStep( step )
	slider:SetWidth( 240 )
	slider:SetHeight( 16 )
	slider:SetScript( "OnValueChanged", function( self )
		self.Val:SetNumber( self:GetValue() )
		PlaySound( "GAMEGENERICBUTTONPRESS" )
	end)
	getglobal(slider:GetName() .. 'Low'):SetText(tostring(min))
	getglobal(slider:GetName() .. 'High'):SetText(tostring(max))
	getglobal(slider:GetName() .. 'Text'):SetText(TextTable[ Name ]); --Sets the "title" text (top-centre of slider).

	slider:SetScript("OnMouseWheel", function(self, delta)
		SliderMW(self, delta)
	end)
	return slider
end

BillsUtils.SliderEditBox = function( Slider , Prefix, Name, Parent )
	local Val
	Val=CreateFrame("EditBox", Prefix..Name.."Value", Parent, "InputBoxTemplate")
	Val:SetFontObject("ChatFontNormal")
	Val:SetTextInsets(0, 0, 3, 3)
	Val:SetPoint("BOTTOMLEFT", Slider , "BOTTOMRIGHT", X , -8)
	Val:SetPoint("RIGHT", -32, 0)
	Val:SetHeight(19)
	Val:SetWidth(32)
	Val:SetNumeric(true)
	Val:SetAutoFocus(false)
	Val:SetMaxLetters( 4 )
	Val.Owner = Slider
	Val:SetScript("OnEnterPressed" , function(self)
		local min, max = self.Owner:GetMinMaxValues()
		local val = self:GetNumber()
		if val >= min and val <= max then
			if val ~= self.Owner:GetValue() then
				self.Owner:SetValue(val)
			end
		else
			self:SetNumber(self.Owner:GetValue())
		end
		self:ClearFocus()
		
	end)
	Val:SetScript("OnEscapePressed", function(self)
		self:SetNumber( self.Owner:GetValue() )
		self:ClearFocus()
	end)
	return Val
end

-- Checks Slider against variable and changes var if needed
BillsUtils.SliderSave = function( Panel, Slider, Table, Key)
	if Key == nil then
		Key = Slider
	end
	local SliderValue = Panel[Slider]:GetValue()
	local SavedValue = Table[Key]
	
	if SliderValue ~= SavedValue then
		Table[Key] = SliderValue
	end
end

-- Mousewheel control for sliders
BillsUtils.SliderMW = function(self, delta)
	local min,max = self:GetMinMaxValues()
	local step = self:GetValueStep()
	local value = self:GetValue()
	
	if (value > min) and (delta < 0) then 
		value = (value - step)
		if (value > min) then 
			self:SetValue( value )
		else
			self:SetValue( min )
		end
	end
	if (value < max) and (delta > 0) then
		value = (value + step)
		if (value < max) then
			self:SetValue( value )
		else
			self:SetValue( max )
		end
	end
end

-- ****************************************************************************
-- * Edit Box Utils                                                           *
-- ****************************************************************************

BillsUtils.EditBox = function( Last, Prefix, Name, Parent, Table, Key, TextTable, TTKey )
	if TTKey == nil then
		TTKey = Key
	end
	
	local EBox
	EBox = CreateFrame( "EditBox", Prefix..Name.."Editbox", Parent, "InputBoxTemplate" )
	EBox:SetFontObject("ChatFontNormal")
	EBox:SetTextInsets(0, 0, 3, 3)
	EBox:SetPoint("TOPLEFT", Last , "TOPLEFT", 0 ,-Y*2 )
	EBox:SetPoint("RIGHT", -32, 0)
	EBox:SetHeight(19)
	EBox:SetWidth(50)
	EBox:SetAutoFocus(false)
	EBox:SetMaxLetters( 254 )
	EBox:SetScript("OnEnterPressed" , function(self)
		self:ClearFocus()
	end)
	EBox:SetScript("OnEscapePressed", function(self)
		self:SetText( Table[Key] )
		me.Default:ClearFocus()
	end)
	
	EBox.Text = Parent:CreateFontString( nil, "ARTWORK", "GameFontHighlightSmall" )
	EBox.Text:SetPoint( "BOTTOMLEFT", EBox , "BOTTOMLEFT", 0 , Y/4 )
	EBox.Text:SetPoint( "RIGHT", -32, 0 )
	EBox.Text:SetHeight( 32 )
	EBox.Text:SetJustifyH( "LEFT" )
	EBox.Text:SetJustifyV( "TOP" )
	EBox.Text:SetText( TextTable[ TTKey ] )
	return EBox
end

-- Frame Event Registration Toggler
BillsUtils.EventToggle = function( Frame, Table, Key, Event, Enable )
	if not(Enable) then
		if Frame:IsEventRegistered(Event) then
			Frame:UnregisterEvent( Event )
		end
		return
	end
	
	if Frame:IsEventRegistered(Event) ~= Table[Key] then
		if Table[Key] then
			Frame:RegisterEvent( Event )
		else
			Frame:UnregisterEvent( Event )
		end
	end
end



-- Line below minimizes mem usage by dropping unused functions
-- 5 mins after loading
BillsUtils.Wait(300, table.wipe, BillsUtils)